# Generated by Django 6.0.2 on 2026-03-12 23:52

import django_pymissive.models.attachment
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_pymissive", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="missivebaseattachment",
            name="attachment_file",
            field=models.FileField(
                blank=True,
                help_text="Leave blank if the attachment is hosted externally",
                null=True,
                storage=django_pymissive.models.attachment.ConfigurableAttachmentStorage(),
                upload_to=django_pymissive.models.attachment._attachment_upload_to,
                verbose_name="Attachment File",
            ),
        ),
    ]
