# Generated by Django 6.0.2 on 2026-03-18 11:53

import django_pymissive.models.attachment
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "django_pymissive",
            "0003_missivebilling_delete_missiverecipientnotification_and_more",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="MissiveProof",
            fields=[],
            options={
                "verbose_name": "Proof",
                "verbose_name_plural": "Proofs",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("django_pymissive.missivebaseattachment",),
        ),
        migrations.AlterModelOptions(
            name="missivecampaign",
            options={
                "ordering": ["-created_at", "subject"],
                "verbose_name": "Campaign",
                "verbose_name_plural": "Campaigns",
            },
        ),
        migrations.AlterField(
            model_name="missivebaseattachment",
            name="attachment_file",
            field=models.FileField(
                blank=True,
                help_text="Leave blank if the attachment is hosted externally",
                max_length=2000,
                null=True,
                storage=django_pymissive.models.attachment.ConfigurableAttachmentStorage(),
                upload_to=django_pymissive.models.attachment._attachment_upload_to,
                verbose_name="Attachment File",
            ),
        ),
        migrations.AlterField(
            model_name="missivebaseattachment",
            name="priority",
            field=models.PositiveIntegerField(
                default=1,
                help_text="Page order. 0=first-document (letter body), others use 1, 2, 3... (0 reserved)",
                verbose_name="Priority",
            ),
        ),
    ]
